/*
 * bufrectpainterarray.cc
 * 
 * Buffered rectangle painter array. Rectangle cache for performant 
 * rectangle drawing in different colors. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file is part of QtXLib (Fast XLib drawing interface for Qt/X11). 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "bufrectpainterarray.h"

#include <stdlib.h>
#include <assert.h>

#include "qtxalloc.h"


namespace QTX
{

void BufferedRectPainterArray::Flush()
{
	for(uint i=0; i<asize; i++)
	{
		BufferedRectPainter *ppa=pa[i];
		if(ppa)
		{  ppa->Flush();  }
	}
}


int BufferedRectPainterArray::Flush(uint pidx)
{
	if(pidx>=asize)  return(1);
	BufferedRectPainter *ppa=pa[pidx];
	if(!ppa)  return(2);
	ppa->Flush();
	return(0);
}


void BufferedRectPainterArray::Setup(uint array_size)
{
	Flush();
	
	if(pa)
	{
		for(uint i=0; i<asize; i++)
		{  delete pa[i];  }
	}
	Free(pa);
	pa=NULL;
	
	asize=array_size;
	if(asize)
	{
		pa=(BufferedRectPainter**)Malloc(asize*sizeof(BufferedRectPainter*));
		for(uint i=0; i<asize; i++)
		{  pa[i]=NULL;  }
	}
}

int BufferedRectPainterArray::Setup(uint pidx,
	const QPaintDevice *pd,uint cache_size,bool fill)
{
	if(pidx>=asize)  return(1);
	
	BufferedRectPainter *ppa=pa[pidx];
	if(ppa)  delete ppa;
	ppa=new BufferedRectPainter(pd,cache_size,fill);
	pa[pidx]=ppa;
	
	return(0);
}

int BufferedRectPainterArray::Setup(uint pidx,
	const QPainter *qp,uint cache_size,bool fill)
{
	if(pidx>=asize)  return(1);
	
	BufferedRectPainter *ppa=pa[pidx];
	if(ppa)  delete ppa;
	ppa=new BufferedRectPainter(qp->device(),cache_size,fill);
	ppa->QCopyGC(*qp,/*brush=*/fill);
	pa[pidx]=ppa;
	
	return(0);
}

}  // end of namespace QTX
