/*
 * ximagewidget.cc
 * 
 * XImage Qt widget (with expose events, etc). 
 * 
 * Copyright (c) 2003 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file is part of QtXLib (Fast XLib drawing interface for Qt/X11). 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "ximagewidget.h"
#include "xpainter.h"

#include <assert.h>
#include <stdlib.h>

#include "qtxalloc.h"



namespace QTX
{

void XImageWidget::PutImage(int x,int y,int w,int h)
{
	if(xim)
	{
		QPaintDevice *pd=this;
		XPutImage(dpy,pd->handle(),gc,xim,x,y,x,y,w,h);
		//fprintf(stderr,"XPutImage(%d,%d,%d,%d)\n",x,y,w,h);
	}
}

void XImageWidget::PutImage()
{
	PutImage(0,0,ImageWidth(),ImageHeight());
}

void XImageWidget::PutImageRow(int yco)
{
	PutImage(0,yco,ImageWidth(),1);
}


void XImageWidget::paintEvent(QPaintEvent *ev)
{
	PutImage(
		ev->rect().x(),ev->rect().y(),
		ev->rect().width(),ev->rect().height());
}


void XImageWidget::CreateImage(int width,int height)
{
	if(xim)
	{
		Free(xim->data);
		xim->data=NULL;
		XDestroyImage(xim);
		xim=NULL;
	}
	if(width>0 && height>0)
	{
		QPaintDevice *pd=this;
		xim=XCreateImage(dpy,(Visual*)pd->x11Visual(),
			/*depth=*/QPaintDevice::x11AppDepth(),
			ZPixmap,0,NULL,width,height,BitmapPad(dpy),0);
		assert(xim);
		assert(!xim->data);
		xim->data=(char*)Malloc(size_t(xim->bytes_per_line)*xim->height);
		
		setFixedSize(width,height);
		resize(width,height);  // to be sure
	}
}


XImageWidget::XImageWidget(int width,int height,
	QWidget *parent,const char *name,WFlags f) : 
	QWidget(parent,name,f)
{
	QPaintDevice *pd=this;
	dpy=pd->x11Display();
	
	xim=NULL;
	
	CreateImage(width,height);
	
	gc=XCreateGC(dpy,pd->handle(),0,NULL);
	QPainter tmp_pnt(this);
	XPainter tmp(&tmp_pnt,1);
	XCopyGC(dpy,tmp.get_gc(),0xffffffffU,gc);
	assert(gc);
	
	if(parent!=NULL && xim)
	{  show();  }
}

XImageWidget::~XImageWidget()
{
	CreateImage(0,0);
	
	if(gc)
	{  XFreeGC(dpy,gc);  gc=NULL;  }
}

}
