/*
 * ximagewidget.h
 * 
 * XImage Qt widget (with expose events, etc). 
 * 
 * Copyright (c) 2003 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file is part of QtXLib (Fast XLib drawing interface for Qt/X11). 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */
 
#ifndef _INC_QTXLIB_XIMAGEWIDGET_H_
#define _INC_QTXLIB_XIMAGEWIDGET_H_

#include <Qt/qwidget.h>
#include <Qt/qevent.h>

//#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

namespace QTX
{

class XImageWidget : public QWidget
{
	private:
		Display *dpy;
		XImage *xim;
		GC gc;
	protected:
		void paintEvent(QPaintEvent *);
	public:
		// Use width=height=0 to create NULL imaage. 
		// Create image lateron with Create(). 
		XImageWidget(int width,int height,
			QWidget *parent=NULL,const char *name=NULL,WFlags f=0);
		~XImageWidget();
		
		// Create image. Destroy current one if needed. 
		// Use width=height=0 for NULL image. 
		void CreateImage(int width,int height);
		
		// Get size: 
		inline int ImageWidth()   {  return(xim ? xim->width : -1);  }
		inline int ImageHeight()  {  return(xim ? xim->height : -1);  }
		inline int ImageDepth()   {  return(xim ? xim->depth : -1);  }
		
		// Get/put pixel; NO checks for speed's sake. 
		inline void _PutPixel(int x,int y,unsigned long color)
			{  XPutPixel(xim,x,y,color);  }
		inline unsigned long _GetPixel(int x,int y)
			{  return(XGetPixel(xim,x,y));  }
		
		// Get/put pixel; with checks. 
		inline void PutPixel(int x,int y,unsigned long color)
		{
			if(xim && x>=0 && x<=xim->width && y>=0 && y<xim->height)
			{  XPutPixel(xim,x,y,color);  }
		}
		inline unsigned long GetPixel(int x,int y)
		{
			return((xim && x>=0 && x<=xim->width && y>=0 && y<xim->height) ? 
				XGetPixel(xim,x,y) : 0);
		}
		
		// Update image, i.e. put it on screen. 
		// Expose events are handeled automatically. 
		void PutImage();  // complete image
		void PutImageRow(int yco);   // complete single scan line (row)
		void PutImage(int x,int y,int w,int h);  // part of image
};

}

#endif  /* _INC_QTXLIB_XIMAGEWIDGET_H_ */
