/*
 * areasetup.cc
 * 
 * Area parameter widget. 
 * 
 * Copyright (c) 2002--2004 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "areasetup.h"

#include <Qt/qspinbox.h>
#include <Qt/qlineedit.h>
#include <Qt/qpushbutton.h>
#include <Qt/qcheckbox.h>
#include <Qt/qlayout.h>
#include <Qt/qlabel.h>
#include <Qt/qvalidator.h>
#include <Qt/qtooltip.h>

#include <stdlib.h>
#include <assert.h>


void AreaSetupDialog::map()
{
	if(isVisible())
	{  hide();  return;  }
	
	AreaSetupPars spar;
	emit get_area_setup(&spar);
	assert(spar.valid);
	
	widthi->setValue(spar.width);
	heighti->setValue(spar.height);
	torus_area->setChecked(spar.torus_area);
	
	show();
}


void AreaSetupDialog::ok_pressed()
{
	AreaSetupPars spar;
	emit get_area_setup(&spar);
	assert(spar.valid);
	
	spar.width=widthi->value();
	spar.height=heighti->value();
	spar.torus_area=torus_area->isChecked();
	
	emit change_area_setup(&spar);
	hide();
}

void AreaSetupDialog::cancel_pressed()
{
	hide();
}


AreaSetupDialog::AreaSetupDialog(QWidget *_parent,const char *name,WFlags f) : 
	QWidget(_parent,name,f)
{
	QPushButton *OK=new QPushButton("&OK",this);
	  connect(OK,SIGNAL(clicked()),this,SLOT(ok_pressed()));
	QPushButton *cancel=new QPushButton("&Cancel",this);
	  connect(cancel,SIGNAL(clicked()),this,SLOT(cancel_pressed()));
	
	QHBoxLayout *buttonlayout=new QHBoxLayout();
	  buttonlayout->addWidget(OK);
	  buttonlayout->addWidget(cancel);
	
	static const char *_areaw_str="Area width (number of\ntrees in X direction).";
	static const char *_areah_str="Area height (number of\ntrees in Y direction).";
	
	QWidget *parent=this;
	widthi=new QSpinBox(1,5000,1,parent);
	  QToolTip::add(widthi,_areaw_str);
	heighti=new QSpinBox(1,5000,1,parent);
	  QToolTip::add(heighti,_areah_str);
	
	QWidget *wtmp;
	QGridLayout *inplayout=new QGridLayout(2,2);
	  inplayout->addWidget(wtmp=new QLabel(widthi,"&Width:",parent),0,0);
	    QToolTip::add(wtmp,_areaw_str);
	  inplayout->addWidget(wtmp=new QLabel(heighti,"&Height:",parent),1,0);
	    QToolTip::add(wtmp,_areah_str);
	  inplayout->addWidget(widthi,0,1);
	  inplayout->addWidget(heighti,1,1);
	
	torus_area=new QCheckBox("&Torus area",parent);
	  QToolTip::add(torus_area,
	    "When enabled, opposite area\n"
	    "borders are connected.");
	
	QVBoxLayout *toplayout=new QVBoxLayout(this,4,4);
	  toplayout->addLayout(inplayout);
	  toplayout->addWidget(torus_area);
	  toplayout->addSpacing(20);
	  toplayout->addLayout(buttonlayout);
}


AreaSetupDialog::~AreaSetupDialog()
{
	
}

